// *********************************************************************
//                    Analog comparator controled via RF
//                             Demo example 
// *********************************************************************
//
// Intended for:
//   HW: - TR-72G + DK-EVAL-04x development kit
//
//   OS: - v4.06G
//
// Description:
//   - this example receives RF commands and controls DAC output (pos. input of Comparator)
//   - use example E03-TR as a transmiter 
//   - use IQRF IDE 4.47 or higher - Terminal (Use Numbers must be checked, see IQRF IDE Help)
//   - import prepared macros file comparator_macros.iqrfmcr to the IQRF IDE
//
//   Commands to be sent from the transmiter: 'C'<DAC output>
//      <DAC output>: 0 ~ 0 V, ..., 16 ~ 1.5 V, ..., 31 ~ 2.9 V
//      For more exact calculation see MCU datasheet.
//      (e.g. "C#016" -> DAC output = 1.5 V)
//
// ---------------------------------------------------------------------
// Online IQRF OS Reference Guide: https://doc.iqrf.org/IQRF-OS-Reference-guide
//
// Copyright (c) IQRF Tech s.r.o.
//
// File:    TR_COMPARATOR.c
// Version: v3.00                                   Revision: 21/06/2021
//
// Revision history:
//   v3.00: 21/06/2021  First release for TR-7xG.
//
// *********************************************************************
// Default IQRF include (modify the path according to your setup)
#include "IQRF.h"

// *********************************************************************
#define RX_FILTER               5

// *********************************************************************
void initComparator();
void userIntRoutine(void);

// *********************************************************************

void APPLICATION()
{
    setLEDR();
    waitDelay(50);                          // Startup indication
    stopLEDR();
    initComparator();
    setRFmode(_WPE | _RX_STD | _TX_STD);
    _checkRFcfg_PQT = 1;                    // Enable preamble quality test
    toutRF = 1;                             // Wait Packed End active so the toutRF can be set to minimum
    _enableUserInterrupt = 1;
    C2IE = 1;                               // Interrupt from Comparator 2 enable
    
    while (1)
    {   
        if (checkRF(RX_FILTER))
        {
            if (RFRXpacket())               
            {
                if (bufferRF[0] == 'C')
                {
                    pulseLEDR();
                    DAC1CON1 = bufferRF[1]; // DAC output control
                }
            }
        }       
    }
}
// *********************************************************************

void initComparator(void)           // Comparator 2 is used with C1 pin as negative input (C12IN0),
{                                   //   and DAC output as positive input.
                                    //   Result is in the CM2CON0.6 bit.

    DACMD = 0;                      // Enable DAC module
    DAC1CON0 = 0b10000000;          // DAC on, Vdd, Vss
    DAC1CON1 = 16;                  // Voltage output (0 - 31), 16 ~ 1,5 V
    
    TRISA.0 = 1;                    // Pin C1 (RA0) as input
    ANSELA.0 = 1;                   // Pin C1 (AN0) as analog input

    CMP2MD = 0;                     // Enable Comparator 2 module
    CM2CON0 = 0b10010100;           // Comparator on, output internal only, output inverted
    CM2CON1 = 0b00000011;           // Interrupt on pos. and neg. edge,
    CM2PSEL = 0b00000.101;          // +input from DAC voltage reference                            
    CM2NSEL = 0b00000.000;          // -input from RA0 pin
    C2IF = 0;                       // Clear interrupt flag
}
// ---------------------------------------------------------------------

// This function is invoked whenever any interrupt occurs (even OS)
// and the _enableUserInterrupt flag is set to 1.
// OS generates an interrupt in the period of system ticks.
// The user interrupt routine must start from address 0x3F00 (__USER_INTERRUPT macro). 
// The user interrupt routine must take less than 50 µs. 
// User interrupts must not be called in period shorter then 800 µs.
// IQRF OS functions except of setINDFx and getINDFx are not allowed.
// IQRF OS variables must not be used.
// Only global or local static user variables are allowed.
// For more information see IQRF OS User's Guide.

#pragma origin __USER_INTERRUPT
#pragma library 0
void userIntRoutine(void)
{                   
    if (C2IF) 
    {
        C2IF = 0;                   // Clear interrupt flag
                                    
        if (CM2CON0.6) _LEDG = 1;   // LEDG is on if neg. input > pos. input
        else           _LEDG = 0;
    }
} 
// *********************************************************************
